package cx.ath.jbzdak.wyklad2.x10mouselisteenrs;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Created by: Jacek Bzdak
 */
public class MouseListenerFrame extends JFrame{

   public MouseListenerFrame() {
      
      PaintLayer paintLayer = new PaintLayer();
      
      add(paintLayer);

      setDefaultCloseOperation(DISPOSE_ON_CLOSE);

      setSize(640, 480);

      paintLayer.addMouseListener(new PaintLayerListener(paintLayer, Color.RED));

      paintLayer.addMouseMotionListener(new PaintLayerMovementListener(paintLayer, Color.BLACK));
      
   }



   public static void main(String[] args) {
      SwingUtilities.invokeLater(new Runnable() {
         public void run() {
            MouseListenerFrame frame = new MouseListenerFrame();
            frame.setVisible(true);
         }
      });
   }
}



class Dot{
   public int x, y, radius;
   
   public Color color;

   Dot() {
   }

   Dot(int x, int y, int radius, Color color) {
      this.x = x;
      this.y = y;
      this.radius = radius;
      this.color = color;
   }
}

class PaintLayerListener extends MouseAdapter{

   final PaintLayer paintLayer;

   final Color color;


   PaintLayerListener(PaintLayer paintLayer, Color color) {
      this.paintLayer = paintLayer;
      this.color = color;
   }


   @Override
   public void mouseClicked(MouseEvent e) {

      paintLayer.addDot(new Dot(e.getX(), e.getY(), 5 * e.getClickCount(), color));
   }
}

class PaintLayerMovementListener extends MouseMotionAdapter{

   final PaintLayer paintLayer;

   final Color color;

   PaintLayerMovementListener(PaintLayer paintLayer, Color color) {
      this.paintLayer = paintLayer;
      this.color = color;
   }

   @Override
   public void mouseDragged(MouseEvent e) {
      paintLayer.addDot(new Dot(e.getX(), e.getY(), 3, color));
   }
}

class PaintLayer extends JPanel {

   List<Dot> dotList = new ArrayList<Dot>();

   Random random = new Random();

   PaintLayer() {
      setOpaque(true);
      setBackground(null);
   }
   
   public void addDot(Dot dot){
      dotList.add(dot);
      this.repaint();
   }
   
   public void removeLastDot(){
      dotList.remove(dotList.size()-1);
   }

   public void removeDots(){
      dotList.clear();
      this.repaint();
   }

   public void addRandomDot(){
      Dot dot = new Dot();
      dot.x = random.nextInt((int) getSize().getWidth());
      dot.y = random.nextInt((int) getSize().getHeight());
      dot.radius = random.nextInt(30);
      dotList.add(dot);
      this.repaint();
   }

   @Override
   public void paint(Graphics g) {

      for (Dot dot : dotList){
         g.setColor(dot.color);
         g.drawOval(dot.x, dot.y, dot.radius, dot.radius);
         g.fillOval(dot.x, dot.y, dot.radius, dot.radius);
      }
   }

}
